package Statusmon::DBStore;

use strict;
use Statusmon::Statusmon;
use DBCapsulator;
use cfgfile;

our( @ISA );

@ISA = qw(Statusmon::Statusmon);

my $nologtill = 0;
my $logged = 0;


sub new
{

    my ($this) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);

    # I want to receive "statuschange" events
    $self->register( "perf" );
    $self->register( "config" );
    $self->sched_frequency( 1 );

    $self->add_feature_provider( "dbstore" );

    return($self);
}


sub start {
    my( $self ) = @_;

    DBCapsulator::config( (new cfgfile( "db.cfg" )) -> read_file() );
    DBCapsulator::schemadir( $BigSister::common::fs{'etc'} );
    my $config = $self->get_feature_provider( "displayconfig" );
    if( $config ) { $self->read_displayconfig( $config->get_current_config() ) };
}


sub read_displayconfig {
    my( $self, $newconfig, $oldconfig ) = @_;

    $self->{"displayconfig"} = $newconfig;
}


        
sub perf {
    my( $self, $cmd ) = @_;

    my( $perftime, $host, $variable, $value ) 
      = @$cmd{"perftime","host","variable","value"};

    my $index = $variable;
    $index =~ s/.*\.(\d+)$/$1/;
    $variable =~ s/(.*)\.\d+$/$1/;
    eval {
	my $hostid = $self->host_id( $host );
	my $varid = $self->variable_id( $variable );
	my $db;
	if( $value =~ /^-?\d+\.?\d*$/ ) {
	    # numeric performance data
	    $self->{"DBStore::perfdb"} = database DBCapsulator( "perf" ) 
	      unless( $self->{"DBStore::perfdb"} );
	    $db = $self->{"DBStore::perfdb"};
	}
	else {
	    # non-numeric ("info") data
	    $self->{"DBStore::perfinfodb"} = database DBCapsulator( "perfinfo" ) unless( $self->{"DBStore::perfinfodb"} );
	    $db = $self->{"DBStore::perfinfodb"};
	}
	die "cannot access performance data database" unless( $db );
	unless( $db->add( {
	    "host" => $hostid,
	    "variable" => $varid,
	    "valindex" => $index,
	    "time" => $perftime,
	    "value" => $value
	} ) ) {
	    $self->{"DBStore::perfinfodb"} = undef;
	    $self->{"DBStore::perfdb"} = undef;
	    die "failed to add entry ($host,$variable,$value) to performance database";
	}
    };

    if( $@ ) {
        if( (! $nologtill) || ($nologtill<time) ) {
	    $nologtill = 0;
	    &BigSister::common::log( "warning", "dbstore: $@" );
	    $logged++;
	    if( $logged > 20 ) {
	        $nologtill = time + 600;
		$logged = 0;
	    }
	}
    }
}


my %hostids;

sub host_id {
    my( $self, $host ) = @_;

    return $self->id_lookup( \%hostids, "host", $host );
}


my %variableids;

sub variable_id {
    my( $self, $variable ) = @_;

    return $self->id_lookup( \%variableids, "variable", $variable );
}


sub id_lookup {
    my( $self, $index, $dbname, $name ) = @_;

    return( $index->{$name} ) if( defined $index->{$name} );
    my $db = database DBCapsulator( $dbname );
    die "failed to access $dbname database" unless( $db );
    unless( %$index ) {
        my $highest = 0;
        $db->search(); 
	while( my $record = $db->getnext() ) {
	    $index->{$record->{"name"}} = $record->{"id"};
	    $highest = $record->{"id"} if( $record->{"id"} > $highest );
	}
	$index->{":highest:"} = $highest;
	return( $index->{$name} ) if( defined $index->{$name} );
    }
    my $id = $index->{":highest:"} + 1;
    my $record = {
        "name" => $name,
	"id" => $index->{":highest:"} + 1
    };
    unless( $db->add( $record ) ) {
        $db->close();
        die "failed to add host to database";
    }
    $db->close();
    $index->{$name} = $index->{":highest:"} = $id;
    return $id;
}


sub event {
    my( $self, $cmd ) = @_;
    my $text;

    print STDERR "Statusmon::Grapher got an event\n" if( $BigSister::common::dl > 6 );
    $self->{"current_event"} = $cmd;
    if( $cmd->{"cmd"} eq "perf" ) {
	$self->perf( $cmd );
    }
    elsif( $cmd->{"cmd"} eq "displayconfig" ) {
	$self->read_displayconfig( $cmd->{"config"}, $cmd->{"previous_config"} );
    }
}




sub run {
    my( $self ) = @_;

    # nothing, at this moment
}


# ======================= DBStore feature provider =========================


1;
