# Oracle monitor
#
# Gert Dewit <gert.dewit@sos.be>
#
# $Id: oracle.pm,v 1.1 2004/06/22 07:14:02 gertdewit Exp $
#
# Check the README file for more information, after installation it should be in
# the $BSHOME/moduleinfo/ directory

package Requester::oracle;

use strict;
use Requester::Requester;

@Requester::oracle::ISA = qw(Requester::Requester);

use MicroTime;

my $oracle = new Requester::oracle();

sub new
{
   my( $this, $args ) = @_;
   my $class = ref($this) || $this;
   my $self = {};
   bless( $self, $class );
   $self->register("oracle");
   return( $self );
}


sub getinfo {
    my( $self, $vars, $indexes ) = @_;

    return undef unless( $indexes );
    my $info = {};

    my $env = $self->{"env"};
    my $user = $env->expand_value( "user" );
    my $password = $env->expand_value( "password" );
    my $database = $env->expand_value( "database" );
    $user = "SCOTT" unless ( $user );
    $password = "TIGER" unless ( $password );

    foreach my $test (@$vars) {
        unless( $test =~ /^(time|status|message|log)\.(\d+)$/ ) {
	    print "   oracle cannot handle $test\n" if( $BigSister::common::dl > 4 );
	    next;
	}
	my $id = $2;
	next if( defined $info->{$id} );
	$info->{$id} = {};

	my $msg= "";
	my $release = "Release unknown";
	my $status = 2;
	my $time = -1;
	my $cmd = 'echo "SHOW RELEASE" | sqlplus -S -L '.$user.'/'.$password.'@'.$database;
	$msg = `$cmd`;
	if($msg =~ /ERROR/) {
	    $status = 0;
	    $info->{$id}->{"message"} = "No connection";
	} else {
	    $release = $msg;
	    my $start = MicroTime::gettimeofday();
	    $cmd = 'echo "select table_name, tablespace_name from user_tables;" | sqlplus -S -L '.$user.'/'.$password.'@'.$database;
	    $msg = `$cmd`;
	    if($msg =~ /ERROR/) {
		$status = 1;
		$info->{$id}->{"message"} = "Query error";
	    } else {
		$status = 3;
		$info->{$id}->{"message"} = "Query OK";
		$time = int( MicroTime::difftime( $start, MicroTime::gettimeofday() ) * 100 + 0.5 ) / 100;
	    }
	}
	$info->{$id}->{"time"} = $time;
	$info->{$id}->{"log"} = $release."\n".$msg;
	$info->{$id}->{"status"} = $status;
    }
    return $info;
}

1;
